package com.starbuds.server.api.demo.util;

import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcRoomRole;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.service.rtc.RtcQueueService;
import com.starbuds.server.common.service.rtc.RtcRoomAuctionService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.RtcUserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

/**
 * Rtc 通用逻辑
 *
 * @author aoe
 * @date 2021/7/28
 */
@Component
public class RtcUtil {

    @Reference(version = "1.0.0")
    private RtcRoomAuctionService rtcRoomAuctionService;

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    private RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    private RtcQueueService rtcQueueService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    /**
     * 申请上麦
     *
     * @param userId
     * @param roomId
     * @param seatNo
     * @return
     */
    public Result sitSeat(Long userId, Long roomId, Integer seatNo, String appVer) {
        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 麦位处理
        if (seatNo == null || seatNo < 0 || seatNo > 8) {
            return new Result(GlobalApiError.OperationFailed, "麦位无效");
        }

        //是否休息
        boolean isRest = !rtcRoomService.isRoomOnline(roomId);
        if (isRest) {
            return new Result(GlobalApiError.OperationFailed, "未开播不能上麦");
        }

        RtcRoom room = rtcRoomService.getRoom(roomId);
        RtcBusinessTypeEnum roomBusinessTypeEnum = RtcBusinessTypeEnum.toEnum(room.getBusinessType());

        if (seatNo == 0) {
            // 判断角色是否主持
            if (rtcRoomService.getRoomRole(roomId, userId) < RtcRoomRole.HOST) {
                return new Result(GlobalApiError.OperationFailed, "您没有主持权限");
            }

            // 判断是否是个播房，只有主播才能上麦
            if (RtcBusinessTypeEnum.RTC_SINGLE.equals(roomBusinessTypeEnum)) {
                if (!room.getUserId().equals(userId)) {
                    return new Result(GlobalApiError.OperationFailed, "您没有上主持麦权限");
                }
            }
        } else {

            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == room.getBusinessType()) {
                if (seatNo > 4) {
                    return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
                }
            }

            // 麦序排队处理
            if (!rtcRoomService.isRoomMicFree(roomId)) {

                // 判断用户是否在麦序中
                if (rtcQueueService.isUserInQueue(roomId, userId)) {
                    return new Result(GlobalApiError.OperationFailed, "您已在队列中，请耐心等待~");
                }

                // 加入麦序
                if (rtcQueueService.addQueueUser(roomId, userId)) {
                    MutableResult mResult = new MutableResult();
                    mResult.put("queue", true);
                    return mResult.build();
                }
            }
        }

        if (rtcSeatService.sitSeat(roomId, userId, seatNo, seatNo == 0, appVer) != seatNo) {
            return new Result(GlobalApiError.OperationFailed, "上麦失败，请稍后再试");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("queue", false);
        mResult.put("authInfo", rtcRoomService.getAgoraAuthInfo(roomId, userId, appVer));
        mResult.put("seatNo", seatNo);
        return mResult.build();
    }

    /**
     * 移除麦序
     *
     * @param userId
     * @param roomId
     */
    public void doRemoveQueueUserHandler(Long userId, Long roomId) {
        if (userId == null) {
            // 移除全部
            rtcQueueService.clearQueue(roomId);
        } else {
            // 判断是否在麦序中
            if (rtcQueueService.isUserInQueue(roomId, userId)) {
                rtcQueueService.removeQueueUser(roomId, userId);
            }
        }
    }

    public Result checkSetUserToSet(Long roomId, Long myUserId, Long userId, Integer seatNo) {
        if (seatNo != null && seatNo > 4) {
            // 判断是不是个播房
            if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomService.getRoomBusinessType(roomId)) {
                return new Result(GlobalApiError.OperationFailed, "个播房间只有四个麦位可用哦~");
            }
        }

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限是否足够
        int roomRole = rtcRoomService.getRoomRole(roomId, myUserId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断用户是否在房间
        if (!rtcUserService.isUserInRoom(roomId, userId)) {
            return new Result(GlobalApiError.OperationFailed, "该用户不在房间");
        }

        // 判断用户是否在麦上
        if (rtcSeatService.getUserSeatNo(roomId, userId) > -1) {
            return new Result(GlobalApiError.OperationFailed, "该用户已在麦上");
        }

        // 判断位置是否空置
        if (seatNo != null && !rtcSeatService.isSeatIdle(roomId, seatNo)) {
            return new Result(GlobalApiError.OperationFailed, "该麦位已被占用");
        }
        return new Result();
    }
}
