package com.starbuds.server.api.demo.v1.impl;

import com.starbuds.server.api.demo.v1.facade.BookDemoFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.demo.BookDO;
import com.starbuds.server.common.demo.BookDemoService;
import com.starbuds.server.common.demo.BookSearchDemoService;
import com.starbuds.server.common.demo.BookVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.sun.org.apache.xpath.internal.operations.Bool;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.*;
import java.util.Random;

/**
 * 书籍接口实现
 *
 * @author gy
 * @date 2023/9/18
 */
@Path("/v1/book")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class BookDemoFaccadeImpl extends BaseFacade implements BookDemoFacade {

    /**
     * 书籍服务
     */
    @Reference(version = "1.0.0")
    BookDemoService bookDemoService;

    /**
     * 书籍搜索服务
     */
    @Reference(version = "1.0.0")
    BookSearchDemoService bookSearchDemoService;

    /**
     * 增加书籍
     *
     * @param bookType
     * @param bookName
     * @param bookAuthor
     * @return
     */
    @POST
    @PermitAll
    @Path("addBook")
    @Override
    public Result addBook(@FormParam("bookType") int bookType,
                          @FormParam("bookName") String bookName,
                          @FormParam("bookAuthor") String bookAuthor) {
        if (bookType == 0) {
            return new Result(GlobalApiError.OperationFailed, "请输入书籍类型");
        }
        if (bookName == null) {
            return new Result(GlobalApiError.OperationFailed, "请输入书籍名");
        }
        if (bookAuthor == null) {
            return new Result(GlobalApiError.OperationFailed, "请输入书籍作者");
        }

        BookDO bookDO = new BookDO();
        bookDO.setBookType(bookType);
        bookDO.setBookId(new Random().nextLong());
        bookDO.setBookName(bookName);
        bookDO.setBookAuthor(bookAuthor);
        long currentTime = System.currentTimeMillis();
        bookDO.setCreateTime(currentTime);
        bookDO.setUpdateTime(currentTime);

        if (bookDemoService.addBook(bookDO)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 删除书籍
     *
     * @param bookId
     * @return
     */
    @POST
    @PermitAll
    @Path("deleteBook")
    @Override
    public Result deleteBook(@FormParam("bookId") long bookId) {
        if (bookId == 0) {
            return new Result(GlobalApiError.OperationFailed, "请传入书籍id");
        }

        if (bookDemoService.deleteBook(bookId)) {
            log.info("不存在还有后续吗");
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 更新书籍
     *
     * @param bookId
     * @param bookName
     * @param bookAuthor
     * @return
     */
    @POST
    @PermitAll
    @Path("updateBook")
    @Override
    public Result updateBook(@FormParam("bookId") long bookId,
                             @FormParam("bookName") String bookName,
                             @FormParam("bookAuthor") String bookAuthor) {
        if (bookId == 0) {
            return new Result(GlobalApiError.OperationFailed, "请传入书籍id");
        }
        if (bookName == null && bookAuthor == null) {
            return new Result(GlobalApiError.OperationFailed, "请输入书籍名或作者");
        }

        if (bookDemoService.updateBook(bookId, bookName, bookAuthor)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取书籍列表
     *
     * @param bookType 为空时获取全部书籍
     * @param bookName 为空时获取全部书籍
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @PermitAll
    @Path("getBookList")
    @Override
    public Result getBookList(@QueryParam("bookType") int bookType,
                              @QueryParam("bookName") String bookName,
                              @QueryParam("pageIndex") int pageIndex,
                              @QueryParam("pageSize") int pageSize) {
        if (pageIndex == 0 || pageSize == 0) {
            pageIndex = 1;
            pageSize = 10;
        }
        PageList<BookVO> pageList = bookDemoService.getBookList(bookType, bookName, pageIndex, pageSize);
        return new Result(pageList);
    }

    /**
     * 获取作者所有的书对应的音乐列表
     *
     * @param bookAuthor
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @PermitAll
    @Path("getBookMusicList")
    @Override
    public Result getBookMusicList(@QueryParam("bookAuthor") String bookAuthor,
                                   @QueryParam("pageIndex") int pageIndex,
                                   @QueryParam("pageSize") int pageSize) {
        if (bookAuthor == null) {
            return new Result(GlobalApiError.OperationFailed, "请输入作者");
        }
        if (pageIndex == 0 || pageSize == 0) {
            pageIndex = 1;
            pageSize = 10;
        }
        PageList<BookVO> pageList = bookDemoService.getBookMusicList(bookAuthor, pageIndex, pageSize);
        return new Result(pageList);
    }

    /**
     * 搜索书籍
     *
     * @param content
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @PermitAll
    @Path("searchBook")
    @Override
    public Result searchBook(@QueryParam("content") String content,
                             @QueryParam("pageIndex") int pageIndex,
                             @QueryParam("pageSize") int pageSize) {
        if (content == null) {
            return new Result(GlobalApiError.OperationFailed, "请输入内容");
        }
        if (pageIndex == 0 || pageSize == 0) {
            pageIndex = 1;
            pageSize = 10;
        }

        PageList<BookVO> bookVOPageList = bookSearchDemoService.searchBook(content, pageIndex, pageSize);
        return new Result(bookVOPageList);
    }
}
